@@ -190,7 +190,7 @@ class Entity(object):
         self.max_cores = max_cores
         self.max_mem = max_mem
         self.max_gpus = max_gpus
-        self.env = env
+        self.env = self.convert_env(env)
         self.params = params
         self.resubmit = resubmit
         self.tpv_tags = TagSetManager.from_dict(tpv_tags or {})
@@ -212,27 +212,32 @@ class Entity(object):
                 setattr(result, k, copy.deepcopy(v, memodict))
         return result
 
-    def process_complex_property(self, prop, context, func):
+    def process_complex_property(self, prop, context, func, stringify=False):
         if isinstance(prop, str):
             return func(prop, context)
         elif isinstance(prop, dict):
-            evaluated_props = {key: self.process_complex_property(childprop, context, func)
+            evaluated_props = {key: self.process_complex_property(childprop, context, func, stringify=stringify)
                                for key, childprop in prop.items()}
             return evaluated_props
         elif isinstance(prop, list):
-            evaluated_props = [self.process_complex_property(childprop, context, func)
+            evaluated_props = [self.process_complex_property(childprop, context, func, stringify=stringify)
                                for childprop in prop]
             return evaluated_props
         else:
-            return prop
+            return str(prop) if stringify else prop  # To handle special case of env vars provided as ints
 
     def compile_complex_property(self, prop):
         return self.process_complex_property(
             prop, None, lambda p, c: self.loader.compile_code_block(p, as_f_string=True))
 
-    def evaluate_complex_property(self, prop, context):
+    def evaluate_complex_property(self, prop, context, stringify=False):
         return self.process_complex_property(
-            prop, context, lambda p, c: self.loader.eval_code_block(p, c, as_f_string=True))
+            prop, context, lambda p, c: self.loader.eval_code_block(p, c, as_f_string=True), stringify=stringify)
+
+    def convert_env(self, env):
+        if isinstance(env, dict):
+            env = [dict(name=k, value=v) for (k, v) in env.items()]
+        return env
 
     def validate(self):
         """
@@ -275,6 +280,16 @@ class Entity(object):
                f"tags={self.tpv_tags}, rank={self.rank[:10] if self.rank else ''}, inherits={self.inherits}, "\
                f"context={self.context}"
 
+    def merge_env_list(self, original, replace):
+        for i, original_elem in enumerate(original):
+            for j, replace_elem in enumerate(replace):
+                if (("name" in replace_elem and original_elem.get("name") == replace_elem["name"])
+                        or original_elem == replace_elem):
+                    original[i] = replace.pop(j)
+                    break
+        original.extend(replace)
+        return original
+
     def override(self, entity):
         if entity.merge_order <= self.merge_order:
             # Use the broader class as a base when copying. Useful in particular for Rules
@@ -292,8 +307,7 @@ class Entity(object):
         new_entity.max_cores = self.max_cores if self.max_cores is not None else entity.max_cores
         new_entity.max_mem = self.max_mem if self.max_mem is not None else entity.max_mem
         new_entity.max_gpus = self.max_gpus if self.max_gpus is not None else entity.max_gpus
-        new_entity.env = copy.copy(entity.env) or {}
-        new_entity.env.update(self.env or {})
+        new_entity.env = self.merge_env_list(copy.deepcopy(entity.env) or [], copy.deepcopy(self.env) or [])
         new_entity.params = copy.copy(entity.params) or {}
         new_entity.params.update(self.params or {})
         new_entity.resubmit = copy.copy(entity.resubmit) or {}
@@ -384,7 +398,7 @@ class Entity(object):
             new_entity.mem = min(new_entity.max_mem, new_entity.mem or 0) if new_entity.max_mem else new_entity.mem
             context['mem'] = new_entity.mem
         if self.env:
-            new_entity.env = self.evaluate_complex_property(self.env, context)
+            new_entity.env = self.evaluate_complex_property(self.env, context, stringify=True)
             context['env'] = new_entity.env
         if self.params:
             new_entity.params = self.evaluate_complex_property(self.params, context)
@@ -488,40 +502,16 @@ class Tool(EntityWithRules):
 
     merge_order = 2
 
-    def __init__(self, loader, id=None, abstract=False, cores=None, mem=None, gpus=None, min_cores=None, min_mem=None,
-                 min_gpus=None, max_cores=None, max_mem=None, max_gpus=None, env=None, params=None, resubmit=None,
-                 tpv_tags=None, rank=None, inherits=None, context=None, rules=None):
-        super().__init__(loader, id=id, abstract=abstract, cores=cores, mem=mem, gpus=gpus, min_cores=min_cores,
-                         min_mem=min_mem, min_gpus=min_gpus, max_cores=max_cores, max_mem=max_mem, max_gpus=max_gpus,
-                         env=env, params=params, resubmit=resubmit, tpv_tags=tpv_tags, rank=rank, inherits=inherits,
-                         context=context, rules=rules)
-
 
 class Role(EntityWithRules):
 
     merge_order = 3
 
-    def __init__(self, loader, id=None, abstract=False, cores=None, mem=None, gpus=None, min_cores=None, min_mem=None,
-                 min_gpus=None, max_cores=None, max_mem=None, max_gpus=None, env=None, params=None, resubmit=None,
-                 tpv_tags=None, rank=None, inherits=None, context=None, rules=None):
-        super().__init__(loader, id=id, abstract=abstract, cores=cores, mem=mem, gpus=gpus, min_cores=min_cores,
-                         min_mem=min_mem, min_gpus=min_gpus, max_cores=max_cores, max_mem=max_mem, max_gpus=max_gpus,
-                         env=env, params=params, resubmit=resubmit, tpv_tags=tpv_tags, rank=rank, inherits=inherits,
-                         context=context, rules=rules)
-
 
 class User(EntityWithRules):
 
     merge_order = 4
 
-    def __init__(self, loader, id=None, abstract=False, cores=None, mem=None, gpus=None, min_cores=None, min_mem=None,
-                 min_gpus=None, max_cores=None, max_mem=None, max_gpus=None, env=None, params=None, resubmit=None,
-                 tpv_tags=None, rank=None, inherits=None, context=None, rules=None):
-        super().__init__(loader, id=id, abstract=abstract, cores=cores, mem=mem, gpus=gpus, min_cores=min_cores,
-                         min_mem=min_mem, min_gpus=min_gpus, max_cores=max_cores, max_mem=max_mem, max_gpus=max_gpus,
-                         env=env, params=params, resubmit=resubmit, tpv_tags=tpv_tags, rank=rank, inherits=inherits,
-                         context=context, rules=rules)
-
 
 class Destination(EntityWithRules):
 
